/*
This software is published under the GNU GPL V3 license.
See http://www.gnu.org/licenses/gpl-3.0.en.html for Details.

Use Arduino IDE 1.6.3 (or a newer one if it works ;-).

0. A fast blinking LED at the HC-06 modul indicates that the power supply is OK.
1. Connect the Due with the programming port to the PC and upload this program (maybe the 
   BTBaudrate has to be adjusted).
2. Link the mobile phone with the HC-06 module, the PIN of the HC-06 is usually 1234.
3. Install the app "Bluetooth spp tools pro" on the mobile phone (Android).
4. Start "Bluetooth spp tools pro" and "Connect" with the HC-06, then select "CMD line mode". 
   The LED at the HC-06 is now continuously lighted.
5. Now you can send numbers between 1 and 128 (micro steps) to the HC-06 and the 2 motors 
   will turn corresponding.

Deutsche Anleitung:

0. LED am HC-06 Modul blinkt schnell, wenn Stromversorgung OK ist.
1. Due ueber Programming Port an PC anschliessen und dieses Prog. hochladen (evtl. BTBaudrate anpassen).
2. In den Handy-Bluetooth Einstellungen koppeln (pairen), PIN vom HC-06 ist normalerweise 1234
3. Auf dem Handy (Android) die App "Bluetooth spp tools pro" installieren.
4. "Bluetooth spp tools pro" starten und "Connect" mit mit HC-06, dann "CMD line mode". 
   Die LED auf dem HC-06 leuchtet jetzt kontinuierlich.
5. Jetzt koennen Zahlen zwischen 1 und 128 (Mikroschritte) an das HC-06 gesendet werden und 
   die 2 Motoren drehen dann entsprechend.

*/

#include "DueTimer.h"

// Bluetooth:
#define BTBaudrate 9600  // Use ansi baud rates: 9600, 19800, 38400,  57600, 115200, 230400
char text[200];
int textPos = 0;
String bt_text = "";
int anz = 0;
byte mikros = 255;
byte multi;
byte tmp;

// Zeiten Geschw. etc.
unsigned long p1ms = 0, p1ms_start = 0; // Programmlaufzeit in 0.1 ms:

// Stepper:
byte dir_pin[8]; // Stepper Dir pins: x, y, z, e0, e1, e2, e3, e4
byte stp_pin[8]; // Stepper Step pins: x, y, z, e0, e1, e2, e3, e4
byte ena_pin[8]; // Stepper enable pins: x, y, z, e0, e1, e2, e3, e4
byte ms1_pin, ms2_pin, ms3_pin; // Stepper Mikroschritt: e5, e6

//Hilfsvariablen:
int ii = 0, jj = 0, kk = 0, ll = 0, mm = 0;


//*******************************************************************************************************************
void setup() {
  delay(10);

  Serial.begin(115200); // Serieller Monitor (PC)
  Serial1.begin(BTBaudrate); // Bluetooth
  
  // Schrittmotoren:
  // X-Achse:
  dir_pin[0] = 23; // Stepper Dir pins: 0=x, 1=y, 2=z, 3=e0, 4=e1, 5=e2
  stp_pin[0] = 24; // Stepper Step pins: x, y, z, e0, e1, e2
  ena_pin[0] = 26; // Stepper enable pins: x, y, z, e0, e1, e2
  // Y-Achse:
  dir_pin[1] = 16; // Stepper Dir pins: 0=x, 1=y, 2=z, 3=e0, 4=e1, 5=e2
  stp_pin[1] = 17; // Stepper Step pins: x, y, z, e0, e1, e2
  ena_pin[1] = 22; // Stepper enable pins: x, y, z, e0, e1, e2
  // Z-Achse:
  dir_pin[2] = 3; // Stepper Dir pins: 0=x, 1=y, 2=z, 3=e0, 4=e1, 5=e2
  stp_pin[2] = 2; // Stepper Step pins: x, y, z, e0, e1, e2
  ena_pin[2] = 15; // Stepper enable pins: x, y, z, e0, e1, e2
  // E1-Achse:
  dir_pin[3] = A6; // Stepper Dir pins: 0=x, 1=y, 2=z, 3=e0, 4=e1, 5=e2
  stp_pin[3] = A7; // Stepper Step pins: x, y, z, e0, e1, e2
  ena_pin[3] = A8; // Stepper enable pins: x, y, z, e0, e1, e2
  // E2-Achse:
  dir_pin[4] = A9; // Stepper Dir pins: 0=x, 1=y, 2=z, 3=e0, 4=e1, 5=e2
  stp_pin[4] = A10; // Stepper Step pins: x, y, z, e0, e1, e2
  ena_pin[4] = A11; // Stepper enable pins: x, y, z, e0, e1, e2
  // E3-Achse:
  dir_pin[5] = 53; // Stepper Dir pins: 0=x, 1=y, 2=z, 3=e0, 4=e1, 5=e2
  stp_pin[5] = 51; // Stepper Step pins: x, y, z, e0, e1, e2
  ena_pin[5] = 49; // Stepper enable pins: x, y, z, e0, e1, e2
  // E4-Achse:
  dir_pin[6] = 33; // Stepper Dir pins: 0=x, 1=y, 2=z, 3=e0, 4=e1, 5=e2
  stp_pin[6] = 35; // Stepper Step pins: x, y, z, e0, e1, e2
  ena_pin[6] = 37; // Stepper enable pins: x, y, z, e0, e1, e2
  // E5-Achse:
  dir_pin[7] = 27; // Stepper Dir pins: 0=x, 1=y, 2=z, 3=e0, 4=e1, 5=e2
  stp_pin[7] = 29; // Stepper Step pins: x, y, z, e0, e1, e2
  ena_pin[7] = 31; // Stepper enable pins: x, y, z, e0, e1, e2
  // PINs setzen:
  for (ii = 0; ii <= 7; ii++){
    pinMode(dir_pin[ii], OUTPUT); 
    pinMode(stp_pin[ii], OUTPUT); 
    pinMode(ena_pin[ii], OUTPUT); 
    digitalWrite(dir_pin[ii], LOW);
    digitalWrite(stp_pin[ii], LOW);
    digitalWrite(ena_pin[ii], LOW);
  }

  ms1_pin = 67; // Mikroschritteinstellung
  ms2_pin = 68;
  ms3_pin = 69;
  pinMode(ms1_pin, OUTPUT); 
  pinMode(ms2_pin, OUTPUT); 
  pinMode(ms3_pin, OUTPUT); 
  digitalWrite(ms1_pin, HIGH);
  digitalWrite(ms2_pin, HIGH);
  digitalWrite(ms3_pin, HIGH);
  multi = 1;

  // Steuerung der Motoren
  p1ms_start = p1ms;
  digitalWrite(ena_pin[6], HIGH); // enable motor
  digitalWrite(ena_pin[7], HIGH); // enable motor

  // Timer-Interrupt starten:
  Timer3.attachInterrupt(aufruf10kHz);
  Timer3.start(100); // Calls every 0.1ms
}


//*******************************************************************************************************************
void mikroschritte() {
  if (mikros < 129) multi = 7;
  if (mikros < 65) multi = 6;
  if (mikros < 33) multi = 5;
  if (mikros < 17) multi = 4;
  if (mikros < 9) multi = 3;
  if (mikros < 5) multi = 2;
  if (mikros < 3) multi = 1;
  if (mikros < 2) multi = 0;
  digitalWrite(ms1_pin, bitRead(multi, 0));
  digitalWrite(ms2_pin, bitRead(multi, 1));
  digitalWrite(ms3_pin, bitRead(multi, 2));
  mikros = 128 >> multi; 
  multi = mikros; // 128 bei Vollschritt, 1 bei 1/128 Mikroschritt
//  Serial.println(multi);
  mikros = 255; // nur einmal machen
}

//*******************************************************************************************************************
void aufruf10kHz(){ // Interrupt für die Motorbewegung
  p1ms++;
  if ((p1ms - p1ms_start) >= multi) {  // Schritt machen
    p1ms_start = p1ms;  // neue Startzeit
    for (jj = 6; jj <= 7; jj++){
      digitalWrite(stp_pin[jj], HIGH);   // Step machen
    }
    delayMicroseconds(1);
    for (jj = 6; jj <= 7; jj++){
      digitalWrite(stp_pin[jj], LOW);   // Step abschliessen
    }
  }
}

//*******************************************************************************************************************
void loop() {
  if (Serial.available() > 0) {
    char c = Serial.read();
    if (c == '\n' || c == '\r') {
      Serial.print("Send command:");
      text[textPos] = 0;
      Serial.println(text);
      for (int i = 0; i < textPos; i++) {
        Serial1.write(text[i]);
      }
      textPos = 0;
    } else {
      text[textPos++] = c;
    }
  }
  if (Serial1.available()) {
    char c = Serial1.read();
    if (c == '\n' || c == '\r') {
//      Serial.print("Got command:");
//      Serial.println(bt_text);
      // Mikroschritte einstellen:
      tmp = bt_text.toInt(); 
      if ((tmp >= 1) && (tmp <= 128)) {
        mikros = tmp;
        mikroschritte();
      }
      bt_text = "";
    } else {
      bt_text += c;
    }
  }
}



















